<?php
/**
 * Mopple storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *      'table'         The name of the tasks table in 'database'.
 *      'charset'       The database's internal charset.</pre>
 *
 * Required by some database implementations:<pre>
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.</pre>
 *
 * The table structure can be created by the scripts/drivers/mopple_foo.sql
 * script.
 *
 * $Horde: mopple/lib/Driver/sql.php,v 1.4 2004/10/08 15:45:27 chuck Exp $
 *
 * @author  Your Name <you@example.com>
 * @version $Revision: 1.4 $
 * @since   Mopple 0.1
 * @package Mopple
 */
class Mopple_Driver_sql extends Mopple_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * Handle for the current database connection.
     *
     * @var object DB $db
     */
    var $_db;

    /**
     * Boolean indicating whether or not we're connected to the SQL server.
     *
     * @var boolean $_connected
     */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Mopple_Driver_sql($params = array())
    {
        $this->_params = $params;
    }

    /**
     * Retrieves one MP3 from the database.
     *
     * @param string $id  The id of the MP3 to retrieve.
     *
     * @return array  The array of MP3 attributes.
     */
    function get($id)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE song_owner = %s and song_id = %d',
                         $this->_params['table'],
                         $this->_db->quote(Auth::getAuth()),
                         $id);

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Mopple_Driver_sql::get(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
        if (is_a($row, 'PEAR_Error')) {
            return $row;
        }
        if (is_null($row)) {
            return PEAR::raiseError(_("Song not found."));
        }

        /* Decode and return the MP3. */
        $row['song_title'] = String::convertCharset($row['song_title'], $this->_params['charset']);
        $row['song_artist'] = String::convertCharset($row['song_artist'], $this->_params['charset']);
        $row['song_album'] = String::convertCharset($row['song_album'], $this->_params['charset']);
        $row['song_idtag'] = unserialize($row['song_idtag']);

        return $row;
    }

    /**
     * Add a MP3 to the backend storage.
     *
     * @param string $title   The title of the MP3.
     * @param string $author  The artist of the MP3.
     * @param string $album   The album of the MP3.
     * @param array $tag      The ID tag of the MP3.
     *
     * @return int  The ID of the new MP3.
     */
    function add($title, $author, $album, $tag = null)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        $id = $this->_db->nextId($this->_params['table']);

        $query = sprintf(
            'INSERT INTO %s (song_id, song_owner, song_title, ' .
            'song_artist, song_album, song_idtag) ' .
            'VALUES (%d, %s, %s, %s, %s, %s)',
            $this->_params['table'],
            $id,
            $this->_db->quote(Auth::getAuth()),
            $this->_db->quote(String::convertCharset($title, NLS::getCharset(), $this->_params['charset'])),
            $this->_db->quote(String::convertCharset($author, NLS::getCharset(), $this->_params['charset'])),
            $this->_db->quote(String::convertCharset($album, NLS::getCharset(), $this->_params['charset'])),
            $this->_db->quote(serialize($tag)));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Mopple_Driver_sql::add(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Attempt the insertion query. */
        $result = $this->_db->query($query);

        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return $id;
    }

    /**
     * Modifies an existing MP3.
     *
     * @param int $id         The MP3 ID.
     * @param string $title   The title of the MP3.
     * @param string $author  The artist of the MP3.
     * @param string $album   The album of the MP3.
     * @param array $tag      The ID tag of the MP3.
     *
     * @return mixed  True on success, PEAR_Error on error.
     */
    function update($id, $title, $artist, $album, $tag = null)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        $query = sprintf('UPDATE %s SET' .
                         ' song_title = %s, ' .
                         ' song_artist = %s, ' .
                         ' song_album = %s, ' .
                         ' song_idtag = %s ' .
                         'WHERE song_owner = %s AND song_id = %d',
                         $this->_params['table'],
                         $this->_db->quote(String::convertCharset($title, NLS::getCharset(), $this->_params['charset'])),
                         $this->_db->quote(String::convertCharset($artist, NLS::getCharset(), $this->_params['charset'])),
                         $this->_db->quote(String::convertCharset($album, NLS::getCharset(), $this->_params['charset'])),
                         $this->_db->quote(serialize($tag)),
                         $this->_db->quote(Auth::getAuth()),
                         (int)$id);

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Mopple_Driver_sql::update(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Attempt the update query. */
        $result = $this->_db->query($query);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return true;
    }

    /**
     * Retrieves the MP3s from the database.
     *
     * @return boolean|PEAR_Error  True on success, PEAR_Error on failure.
     */
    function retrieve()
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE song_owner = %s',
                         $this->_params['table'], $this->_db->quote(Auth::getAuth()));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Mopple_Driver_sql::retrieve(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (isset($result) && !is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_a($row, 'PEAR_Error')) {
                return $row;
            }

            /* Store the retrieved values in the $_mp3s variable. */
            $this->_mp3s = array();
            while ($row && !is_a($row, 'PEAR_Error')) {
                /* Add this new task to the $tasks list. */
                $this->_mp3s[] = $row;

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }
            $result->free();
        } else {
            return $result;
        }

        return true;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'storage',
                array('phptype', 'hostspec', 'username', 'database', 'charset', 'table'));

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                      array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

    /**
     * Disconnects from the SQL server and cleans up the connection.
     *
     * @return boolean  True on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            return $this->_db->disconnect();
        }

        return true;
    }

}
